# -*- coding: utf-8 -*-

import os, logging, sys
from functools import partial

from qgis.gui import *
from qgis.core import *
from qgis.utils import iface
from PyQt5 import QtGui, QtWidgets, uic, QtXml
from PyQt5.QtCore import *

from .rdi_param import rdiParam
from .rdi_tools import quote, makeUniqueIdent, clearLayout
from .rdi_trade import rdiTrade
from .rdi_postgis import tmpQuery
from .rdi_async import mGlobalTask

from .rdi_meta import rdiMeta


class rdiMask():
    def __init__(self, parent):
        self.dock = parent
        self.meta = rdiMeta()
        self.initData()
        self.choice = "qgis"
        self.table = None
        self.layer = None
        self.trade = rdiTrade(self)
        self.selected = False
        self.postgres = False
         
    def recupIni(self):
        mask = self.meta.readIni("mask")
        if mask is not None and self.dock.psql.isTable(mask):
            self.table = mask
            self.affLayer()
        else:
            self.table = None
    
    def paint(self, layout):
        l = QtWidgets.QLabel("")
        layout.addWidget(l)
        w = QtWidgets.QGroupBox("Masque de travail")
        layout.addWidget(w)
        vbox = QtWidgets.QVBoxLayout(w)
        vbox.setSpacing(2)
        # first = None
        # for k,v in self.options.items():
            # b = QtWidgets.QRadioButton(v, w)
            # b.key = k
            # b.clicked.connect(partial(self.toggleChoice, b))
            # vbox.addWidget(b)
            # if self.choice==k or self.choice==None and not first:
                # first = b
            # if k!="qgis":
                # b.setEnabled(False)
        self.layout = vbox
        
        if self.table!=None:
            self.makeChoosen()
        else:
            self.makeToChoose()
        
        # if first is not None:        
            # first.click()
    
    def toggleChoice(self, b):
        self.choice = b.key
    
    def makeToChoose(self):
        layout = self.layout
        clearLayout(layout)
        l = QtWidgets.QLabel("Aucun masque choisi")
        font = QtGui.QFont()
        font.setItalic(True)
        l.setFont(font)
        layout.addWidget(l)
        w = QtWidgets.QWidget()
        layout.addWidget(w)
        form = QtWidgets.QFormLayout(w)
        form.setContentsMargins(0,10,0,0) 
        combo = QgsMapLayerComboBox()
        combo.setFilters(QgsMapLayerProxyModel.PolygonLayer)
        # if not self.postgres:
            # combo.setExcludedProviders(['postgres'])
        form.addRow("Couche", combo)
        # cb = QtWidgets.QCheckBox("Inclure source Postgres")
        # if self.postgres:
            # cb.setChecked(True)
        # cb.stateChanged.connect(partial(self.actuComboLayer, combo, cb))
        # form.addRow("", cb)
        cb = QtWidgets.QCheckBox("Entités sélectionnées uniquement")
        if self.selected:
            cb.setChecked(True)
        cb.stateChanged.connect(self.actuSelected)
        form.addRow("", cb)
        self.selectedCB = cb
        self.comboQgis = combo
        butt = QtWidgets.QPushButton("Charger")
        butt.setFixedSize(130, 30)
        layout.addWidget(butt)
        layout.setAlignment(butt, Qt.AlignCenter)
        butt.clicked.connect(self.set)
        self.butt = butt
    
    def makeChoosen(self):
        layout = self.layout
        clearLayout(layout)
        
        hbox = QtWidgets.QHBoxLayout()
        layout.addLayout(hbox)
        
        l = QtWidgets.QLabel("Masque choisi")
        font = QtGui.QFont()
        font.setItalic(True)
        l.setFont(font)
        hbox.addWidget(l)
        
        ico = QtGui.QIcon(os.path.join(self.dock.imageFolder,'refresh.png'))
        butt = QtWidgets.QPushButton(ico, '')
        butt.setToolTip("Actualiser")
        butt.setMaximumWidth(20)
        hbox.addWidget(butt)
        butt.clicked.connect(self.affLayer)
        
        l = QtWidgets.QLabel()
        l.setMaximumWidth(100)
        ico = QtGui.QIcon(self.getSVG())
        l.setPixmap(ico.pixmap(100, 100))
        layout.addWidget(l)
        layout.setAlignment(l, Qt.AlignCenter)
        butt = QtWidgets.QPushButton("Supprimer")
        butt.setFixedSize(130, 30)
        layout.addWidget(butt)
        layout.setAlignment(butt, Qt.AlignCenter)
        butt.clicked.connect(self.delete)
        self.butt = butt

    def makeTmp(self):
        tmp = os.path.join(os.path.dirname(__file__), 'svg', 'tmp')
        if not os.path.isdir(tmp):
            os.mkdir(tmp)
    
    def getSVG(self):
        res = self.dock.psql.getSVG(self.table, maxDecimalDigits=0)
        try:
            info = res[0]
            path = info.get('poly')
            box = list(map(float, info.get('box').replace("(","").replace(")","").split(",")))
            w = max((box[0]-box[2]),(box[1]-box[3]))/50
            vb = f"{int(box[2])-2*w} {-int(box[1])-2*w} {int(box[0]-box[2])+4*w} {int(box[1]-box[3])+4*w}"
            
        except:
            path = ""
            vb = "0 0 0 0"
            w = 1
        svg = f''
        svg += f'<svg version="1.1" viewBox="{vb}" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">'
        svg += f'<g fill="#bbbbbb" fill-opacity="0.2" stroke-width="{w}" stroke="#888888">'
        svg += f'<path d="{path}" />'
        svg += f'</g></svg>' 
        self.makeTmp()
        file = os.path.join(os.path.dirname(__file__), 'svg', 'tmp', '_tmp_mask.svg')
        f = QFile(file)
        if not f.open(QIODevice.WriteOnly | QIODevice.Text):
            return
        out = QTextStream(f)
        out << svg 
        f.close()
        return file
    
    def actuComboLayer(self, combo, cb):
        list = ['postgres']
        self.postgres = False
        if cb.isChecked():
            list = []
            self.postgres = True
        combo.setExcludedProviders(list)
        
    def actuSelected(self, cb):
        self.selected = self.selectedCB.isChecked()
  
    def delete(self):
        self.butt.setEnabled(False)
        self.butt.setText("Suppression...")
        QtWidgets.QApplication.processEvents()
        sql = f"DROP TABLE IF EXISTS {self.table}; DELETE FROM {self.dock.psql.admin('tableTmp')} WHERE tablename='{self.table}';"   
        self.dock.psql.sqlSend(sql)
        self.table = None
        self.dock.psql.actuMask(self.table)
        self.makeToChoose()
        self.dock.actuIcoTabFil()
        self.dock.actuTabs(['layer'])

    def set(self):
        if self.choice=='qgis':
            layer = self.comboQgis.currentLayer()
            if layer is None:
                return
            self.butt.setEnabled(False)
            self.butt.setText("Chargement...")
            table = self.dock.psql.tmpPrefixe + makeUniqueIdent()
            schema = self.dock.psql.PGschema  
            self.insert = self.trade.createTableFromFeatures(layer, table, schema, self.fusionPolygon, selected=self.selectedCB.isChecked())

    def fusionPolygon(self, full):
        if not self.dock.psql.isTable(self.insert):
            self.butt.setEnabled(True)
            return
        geom = self.dock.psql.getGeometryField(self.insert)
        sql = f"SELECT 1 as id, ST_Union({quote(geom)}) as geom FROM {self.insert}"
        q = tmpQuery(self.dock.psql, sql)
        q.setPkey('id')
        q.addBD(enjeu="", alea="", requete=f"mask{self.insert}")
        task = mGlobalTask("Fusion masque")
        q.description = f"union entity"
        task.addSubTask(q)
        task.setCallback(self.endInsertion, q.table)
        self.dock.psql.manager.add(task)
  
    def endInsertion(self, table):
        sql = f"DROP TABLE IF EXISTS {self.insert};"   
        self.dock.psql.sqlSend(sql)
        if not self.dock.psql.isTable(table):
            self.butt.setEnabled(True)
            return
        self.table = table
        self.meta.writeIni("mask", self.table)
        self.dock.psql.actuMask(self.table)
        self.makeChoosen()
        self.dock.actuIcoTabFil()
        self.dock.actuTabs(['layer'])
    
    def affLayer(self):
        if self.dock.param.get('maskWorking') and self.table is not None:
            self.dock.psql.getLayerAsync(self.table, f"{self.table}_mask", callback2=self.affLayerEnd)
        else:
            self.layer = self.dock.dyn.removeMapLayer(self)
            
    def affLayerEnd(self, result):
        vlayer = result[0]
        geomType = result[1]
        if not vlayer:
            return
        self.vlayer = vlayer
        self.layer = self.dock.dyn.addInGroup(self, 'mask')
        self.layerId = self.layer.layerId()
        p = self.dock.param

        symbol = QgsFillSymbol().createSimple({'color':p.get('maskFill'), 'stroke':p.get('maskStroke'), 'stroke-width':str(p.get('maskWidth'))})
        c = p.get('maskBurst')
        if not isinstance(c, list) or len(c)<2:
            c = ["#aa000000","#88ffffff"]
        sl = QgsShapeburstFillSymbolLayer(color=QtGui.QColor(c[0]), color2=QtGui.QColor(c[1]), blurRadius=15, useWholeShape=False, maxDistance=10)
        symbol.changeSymbolLayer(0, sl)
        renderer = QgsSingleSymbolRenderer(symbol)
        
        r = QgsInvertedPolygonRenderer(renderer)
        # r.setPreprocessingEnabled(True)
        self.vlayer.setRenderer(r) 
    
    def initData(self):
        self.options = {
            'draw': "Dessiner une emprise",
            'qgis': "Choisir dans Qgis",
            'file': "Choisir un fichier",
        }